#include "elf.h"


elf_t kernel_elf;   //global elf



elf_t elf_from_multiboot (multiboot_t *mb)
{
  int i;
  elf_t elf;
  //get section header
  elf_section_header_t *sh = (elf_section_header_t*)mb->addr;
  //get section header string table
  uint32_t shstrtab = sh[mb->shndx].addr;

  for (i = 0; i < mb->num; i++)
  {
    const char *name = (const char *) (shstrtab + sh[i].name);  //base + offset
    if (!strcmp (name, ".strtab"))
    {
      elf.strtab = (const char *)sh[i].addr;
      elf.strtabsz = sh[i].size;
    }
    if (!strcmp (name, ".symtab"))
    {
      elf.symtab = (elf_symbol_t*)sh[i].addr;
      elf.symtabsz = sh[i].size;
    }
  }
  return elf;
}

const char* elf_lookup_symbol(uint32_t addr, elf_t *elf)
{
    int i;
    int sz = elf->symtabsz/sizeof(elf_symbol_t);
    for(i=0; i<sz; i++)
    {
        if(ELF32_ST_TYPE(elf->symtab[i].info)!= 0x2)    //symbol table
            continue;

        if((addr>=elf->symtab[i].value) &&
           (addr<(elf->symtab[i].value + elf->symtab[i].size)))
        {
            const char *name = (const char*)((uint32_t)elf->strtab + elf->symtab[i].name);
            return name;
        }
    }
}

void init_elf(multiboot_t *mb)
{
   kernel_elf = elf_from_multiboot(mb);
}
