//
#ifndef ELF_H
#define ELF_H

#include "common.h"
#include "multiboot.h"

#define ELF32_ST_TYPE(i) ((i)&0xf)

typedef struct
{
  uint32_t name;    // section name index(string table)
  uint32_t type;    // 2:SHT_SYMTAB 3:SHT_STRTAB
  uint32_t flags;
  uint32_t addr;    // section memory address
  uint32_t offset;
  uint32_t size;    //section size
  uint32_t link;
  uint32_t info;
  uint32_t addralign;
  uint32_t entsize; //every entry size
} __attribute__((packed)) elf_section_header_t;


typedef struct
{
  uint32_t name;    //symbol name(index)
  uint32_t value;
  uint32_t size;   //object size
  uint8_t  info;
  uint8_t  other;
  uint16_t shndx;
} __attribute__((packed)) elf_symbol_t;


//only intereste of symbol table(debug info) and string table(string info)
typedef struct
{
  elf_symbol_t *symtab;   //symbol table array
  uint32_t      symtabsz; // size
  const char   *strtab;   // string table
  uint32_t      strtabsz; // string table size
} elf_t;

// Takes a multiboot structure and returns an elf structure containing the symbol information.
elf_t elf_from_multiboot (multiboot_t *mb);

// Looks up a symbol by address.
const char *elf_lookup_symbol (uint32_t addr, elf_t *elf);

void init_elf(multiboot_t *mb);


#endif
