// gdt
#include "gdt.h"
#include "task.h"

//access asm function
extern void gdt_flush(uint32_t);
extern void tss_flush();


//internal function prototypes
static void gdt_set_gate(uint32_t,uint32_t,uint32_t,uint8_t,uint8_t);
static void write_tss(uint32_t num, uint16_t ss0, uint32_t esp0);
//The GDT itself
gdt_entry_t gdt_entries[6];

//Pointer structure to give to the CPU
gdt_ptr_t gdt_ptr;

tss_entry_t tss_entry;

static void gdt_set_gate(uint32_t num, uint32_t base, uint32_t limit, uint8_t access, uint8_t gran)
{
    gdt_entries[num].base_low = (base & 0xFFFF);
    gdt_entries[num].base_middle = (base>>16) & 0xFF;
    gdt_entries[num].base_high = (base>>24) & 0xFF;

    gdt_entries[num].limit_low   = (limit & 0xFFFF);
    gdt_entries[num].granularity = (limit >> 16) & 0x0F;

    gdt_entries[num].granularity |= gran & 0xF0;
    gdt_entries[num].access      = access;
}

void init_gdt ()
{
  // The limit is the last valid byte from the start of the GDT - i.e. the size of the GDT - 1.
  gdt_ptr.limit = sizeof (gdt_entry_t) * 6 - 1;
  gdt_ptr.base = (uint32_t) &gdt_entries;

  gdt_set_gate (0, 0, 0, 0, 0);             // Null segment.
  gdt_set_gate (1, 0, 0xFFFFFFFF, 0x9A, 0xCF); // Code segment.
  gdt_set_gate (2, 0, 0xFFFFFFFF, 0x92, 0xCF); // Data segment.
  gdt_set_gate (3, 0, 0xFFFFFFFF, 0xFA, 0xCF); // User mode code segment
  gdt_set_gate (4, 0, 0xFFFFFFFF, 0xF2, 0xCF); // User mode data segment

  write_tss(5, 0x10, 0x0);
  // Inform the CPU about our GDT.
  gdt_flush ((uint32_t) &gdt_ptr);
  tss_flush();
}

static void write_tss(uint32_t num, uint16_t ss0, uint32_t esp0)
{
    uint32_t base = (uint32_t)&tss_entry;
    uint32_t limit = base + sizeof(tss_entry);
    //uint32_t limit = sizeof(tss_entry)-1;
    gdt_set_gate(num, base, limit, 0xE9, 0x00); //ring3
    memset(&tss_entry, 0, sizeof(tss_entry));

    tss_entry.ss0 = ss0;    //kernel stack
    tss_entry.esp0 = esp0;
    tss_entry.cs = 0x0b;    //0x8 | 3
    tss_entry.ss = tss_entry.ds = tss_entry.es = tss_entry.fs = tss_entry.gs = 0x13;
}

void set_kernel_stack(uint32_t stack)
{
    tss_entry.esp0 = stack;
}
