;idt.asm --

global idt_flush

idt_flush:
    mov eax, [esp+4]
    lidt [eax]
    ret
.end:

%macro ISR_NOERRCODE 1
    global isr%1
    isr%1:
        cli
        push 0
        push %1
        jmp isr_common_stub
%endmacro

%macro ISR_ERRCODE 1
  global isr%1
  isr%1:
        cli                         ; Disable interrupts.
        push %1                     ; Push the interrupt number
        jmp isr_common_stub
%endmacro

%assign i 0
%rep 8
    ISR_NOERRCODE i
    %assign i i+1
%endrep

ISR_ERRCODE 8

ISR_NOERRCODE 9

%assign i 10
%rep 5
    ISR_ERRCODE i
    %assign i i+1
%endrep

%rep 17
    ISR_NOERRCODE i
    %assign i i+1
%endrep

ISR_NOERRCODE 255

extern isr_handler

global isr_common_stub

isr_common_stub:
    pusha

    mov ax, ds
    push eax

    mov ax, 0x10
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax
    mov ss, ax

    push esp
    call isr_handler
    add esp, 4

    pop ebx                  ; Reload the original data segment descriptor
    mov ds, bx
    mov es, bx
    mov fs, bx
    mov gs, bx
    mov ss, bx

    popa                     ; Pops edi,esi,ebp...
    add esp, 8               ; Cleans up the pushed error code and pushed ISR number
    iret                     ; pops 5 things at once: CS, EIP, EFLAGS, SS, and ESP
.end:



; This macro creates a stub for an IRQ - the first parameter is
; the IRQ number, the second is the ISR number it is remapped to.
%macro IRQ 2
  global irq%1
  irq%1:
    cli
    push byte 0
    push byte %2
    jmp irq_common_stub     ;irq_common_stub
%endmacro

extern irq_handler

irq_common_stub:
   pusha                    ; Pushes edi,esi,ebp,esp,ebx,edx,ecx,eax

   mov ax, ds               ; Lower 16-bits of eax = ds.
   push eax                 ; save the data segment descriptor

   mov ax, 0x10  ; load the kernel data segment descriptor
   mov ds, ax
   mov es, ax
   mov fs, ax
   mov gs, ax
   mov ss, ax

   push esp
   call irq_handler
   add esp, 4

   pop ebx                  ; reload the original data segment descriptor
   mov ds, bx
   mov es, bx
   mov fs, bx
   mov gs, bx
   mov ss, bx

   popa                     ; Pops edi,esi,ebp...
   add esp, 8     ; Cleans up the pushed error code and pushed ISR number
   sti
   iret           ; pops 5 things at once: CS, EIP, EFLAGS, SS, and ESP


%assign i 0
%assign j 32
%rep 16
    IRQ i, j
    %assign i i+1
    %assign j j+1
%endrep
