// main.c -- Define C-code kernel entry point


#include "common.h"
#include "timer.h"
#include "elf.h"
#include "panic.h"
#include "pmm.h"
#include "vmm.h"
#include "heap.h"
#include "multiboot.h"
#include "printk.h"
#include "syscall.h"

void switch_to_user_mode();
extern void set_kernel_stack(uint32_t stack);
void init_memory(multiboot_t *mboot_ptr)
{
    mmap_entry_t *mmap = (mmap_entry_t*)mboot_ptr->mmap_addr;
    size_t sz = mboot_ptr->mmap_length / (mmap[0].size+4);

    size_t i;
    for(i=0; i<sz; i++)
    {
        int base = mmap[i].base_addr_low;
        int len = mmap[i].length_low;

        if(mmap[i].type == 1)
        {

            uint32_t j;
            //printk("%d ",base);
            monitor_write_hex(base);
            monitor_write(" ");
            for(j=base; j<base+len; j+=0x1000)
                pmm_free_page(j);
        }
         //mmap->size doesn't contain the size
    }
    monitor_write("\n");
}

void enable_interrupts()
{
    asm volatile("popl %eax; \
        orl $0x200, %eax; \
        pushl %eax;");
}

extern void set_kernel_stack(uint32_t stack);
void switch_to_user_modes()
{
    uint32_t esp;
    asm volatile("mov %%esp, %0":"=r"(esp));
    set_kernel_stack(esp);
    //monitor_write_hex(cs);
    // Set up our kernel stack.
    //set_kernel_stack(current_task->kernel_stack+KERNEL_STACK_SIZE);

    // Set up a stack structure for switching to user mode.
    asm volatile("  \
      cli; \
      mov $0x23, %ax; \
      mov %ax, %ds; \
      mov %ax, %es; \
      mov %ax, %fs; \
      mov %ax, %gs; \
                    \
       \
      mov %esp, %eax; \
      pushl $0x23; \
      pushl %eax; \
      pushf; \
      pushl $0x1B; \
      push $1f; \
      iret; \
    1: \
      ");

      //enable_interrupts();
}

int kernel_main(uint32_t magic, multiboot_t *mboot_ptr)
{
    init_gdt();
    init_idt();
    init_elf(mboot_ptr);

    printk_clear();

    //asm volatile("int $0x3");
    //init_timer(1);


    //???0x100000  mboot_ptr->mem_upper
    init_pmm(0x400000);
    init_vmm();
    init_heap();
    init_memory(mboot_ptr);

    asm volatile("sti");

	switch_to_user_modes();
    syscall_monitor_write("Hello, user world!\n");
    //syscall_monitor_write_hex("32");
    //monitor_write_hex(esp);
	//printk("%x",magic);
    return 0xDEADBEEF;
}


