#include "monitor.h"

uint8_t cursor_x = 0;
uint8_t cursor_y = 0;
uint16_t *video_memory = (uint16_t*)0xB8000;

static void move_cursor()
{
    uint16_t cursorLocation = cursor_y*80+cursor_x;
    outb(0x3D4,14); //Tell the VGA send the high cursor byte
    outb(0x3D5,cursorLocation>>8);  //send the high cursor byte

    outb(0x3D4,15); //Tell the VGA send the low cursor byte
    outb(0x3D5,cursorLocation);  //send the low cursor byte
    //16 to 8
}

static void scroll()
{
    // 15-12:bg     11-8:fg     7-0:char
    uint8_t attributeByte = (0<<4) | (15&0x0F);
    uint16_t blank = 0x20 /* space */ | (attributeByte<<8);

    if(cursor_y>=25)
    {
        //all line(but last) scroll up one line
        int i;
        for(i=0; i<80*24; i++)
            video_memory[i] = video_memory[i+80];

        //the last line
        for(i=24*80; i<25*80; i++)
            video_memory[i] = blank;

        //the cursor should on the last line
        cursor_y = 24;
    }
}

void monitor_put(char c)
{
    uint8_t backColour = 0;
    uint8_t foreColour = 15;

    uint8_t attributeByte = (backColour<<4) | (foreColour&0x0F);
    uint16_t attribute = attributeByte << 8;

    //backspace
    if(c==0x08 && cursor_x)
        cursor_x--;
    //tab
    else if(c==0x09)
        cursor_x = (cursor_x+8) & ~(8-1);
    //return
    else if(c=='\r')
        cursor_x = 0;
    else if(c=='\n'){
        cursor_x = 0;
        cursor_y++;
    }
    //printable character
    else if(c>=' '){
        video_memory[cursor_y*80+cursor_x] = c|attribute;
        cursor_x++;
    }

    //change new line
    if(cursor_x>=80){
        cursor_x = 0;
        cursor_y++;
    }

    scroll();
    move_cursor();
}

void monitor_clear()
{
    uint8_t attributeByte = (0<<4) | (15 & 0x0F);
    uint16_t blank = 0x20 | (attributeByte<<8);

    int i;
    for(i=0; i<80*25; i++)
        video_memory[i] = blank;

    cursor_x = 0;
    cursor_y = 0;
    move_cursor();
}

void monitor_write(char *c)
{
    while(*c)
        monitor_put((*c++));
}

void monitor_write_hex(uint32_t n)
{
  int tmp;
  char noZeroes = 1;

  monitor_write("0x");

  int i;
  for (i = 28; i >= 0; i -= 4)
  {
    tmp = (n >> i) & 0xF;
    if (tmp == 0 && noZeroes != 0)
      continue;

    noZeroes = 0;
    if (tmp >= 0xA)
      monitor_put (tmp-0xA+'a' );
    else
      monitor_put( tmp+'0' );
  }
}

void monitor_write_dec(uint32_t n)
{
  if (n == 0)
  {
    monitor_put('0');
    return;
  }

  uint32_t acc = n;
  char c[32];
  int i = 0;
  while (acc > 0)
  {
      c[i] = '0' + acc%10;
      acc /= 10;
      i++;
  }
  c[i] = 0;

  char c2[32];
  c2[i--] = 0;
  int j = 0;
  while(i >= 0)
    c2[i--] = c[j++];
  monitor_write(c2);
}


