// multiboot.h -- Declares the multiboot info structure.
//                Made for JamesM's tutorials <www.jamesmolloy.co.uk>

#ifndef MULTIBOOT_H
#define MULTIBOOT_H

#include "common.h"

#define MULTIBOOT_FLAG_MEM     0x001
#define MULTIBOOT_FLAG_DEVICE  0x002
#define MULTIBOOT_FLAG_CMDLINE 0x004
#define MULTIBOOT_FLAG_MODS    0x008
#define MULTIBOOT_FLAG_AOUT    0x010
#define MULTIBOOT_FLAG_ELF     0x020
#define MULTIBOOT_FLAG_MMAP    0x040
#define MULTIBOOT_FLAG_CONFIG  0x080
#define MULTIBOOT_FLAG_LOADER  0x100
#define MULTIBOOT_FLAG_APM     0x200
#define MULTIBOOT_FLAG_VBE     0x400

typedef struct
{
  uint32_t flags;
  uint32_t mem_lower;   //kb
  uint32_t mem_upper;
  uint32_t boot_device;
  uint32_t cmdline;
  uint32_t mods_count;
  uint32_t mods_addr;
  uint32_t num;     //the number of entry
  uint32_t size;    //the size of each entry
  uint32_t addr;    //section header addr
  uint32_t shndx;   //the index of string tabel on section header array
  uint32_t mmap_length;
  uint32_t mmap_addr;   // memory map address,存放内存信息缓冲区地址
  uint32_t drives_length;
  uint32_t drives_addr;
  uint32_t config_table;
  uint32_t boot_loader_name;
  uint32_t apm_table;
  uint32_t vbe_control_info;
  uint32_t vbe_mode_info;
  uint32_t vbe_mode;
  uint32_t vbe_interface_seg;
  uint32_t vbe_interface_off;
  uint32_t vbe_interface_len;
} __attribute__((packed)) multiboot_t;

//memory region
typedef struct
{
  uint32_t size;    // the size of the structure
  uint32_t base_addr_low;   //starting address
  uint32_t base_addr_high;  //32为系统用不到，访问不到4G以上地址
  uint32_t length_low;      //size of the region
  uint32_t length_high;     // 32为系统用不到，访问不到4G以上地址
  uint32_t type;    //1: indicate available
} __attribute__((packed)) mmap_entry_t;

#endif
